/*
 * Driver include for the RICOH5B Tuner chip.
 *
 * Copyright 2015, 2016 SONY Corporation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef  _RICOH5B_CMD_MANAGER_H
#define  _RICOH5B_CMD_MANAGER_H

/******************************************************************************/
/* include file                                                               */
/******************************************************************************/
#include <linux/i2c.h>
#include "ricoh5b_define.h"

/* Definition of Non Mask for Write Register */
#define RICOH5B_REG_NON_MASK      0xFF

/******************************************************************************/
/* enumerated/structured type definition                                      */
/******************************************************************************/
typedef struct {
  unsigned char address;
  unsigned char mask;
  unsigned char data;
} RICOH5B_REG;

/******************************************************************************/
/* macro function definition                                                  */
/******************************************************************************/
#define RICOH5B_GET_UPPER_BYTE(X)              ((unsigned char)((X) >> 8))     /* 2byte data upper level */
#define RICOH5B_GET_LOWER_BYTE(X)              ((unsigned char)(X))            /* 2byte data lower level */

/******************************************************************************/
/* function prototype declaration                                             */
/******************************************************************************/

/* ram initialize */
void ricoh5b_cmd_initialize(void);

void ricoh5b_cmd_regist_get_immunity_measure(RICOH5B_IMMUNITY (*func)(void));

/* getter */
RICOH5B_IMMUNITY  ricoh5b_cmd_get_immunity_measure(void);

/* command sequence */
void ricoh5b_cmd_set_sequence_timer(unsigned long timeout_ms);

int ricoh5b_cmd_read_byte(unsigned char reg_addr, unsigned char *buf);
int ricoh5b_cmd_write_byte(unsigned char reg_addr, unsigned char *buf, size_t count);
int ricoh5b_cmd_write_bit(unsigned char reg_addr, unsigned char data, unsigned char mask);
int ricoh5b_cmd_write_sequencial(RICOH5B_REG *tbl, unsigned char count);

#endif  /* _RICOH5B_CMD_MANAGER_H */
/* end of ricoh5b_cmd_manager.h */
